<?php
//******************************************************************************
//                       CLASS TO GENERATE THE PDF
//******************************************************************************
require_once('include.constants.php');
//Include the FPDF-Class just in case it would be needed
//require_once('fpdf.php'); //To be able to use HTML-Colors FPDF is not directly included but a derived class.
require_once('htmlcolor.php');


class bm_pdf extends PDF_HTMLColor
{
	/*
	This is the container for the finished PDF, it will be written in here as string. 
	The class bm_emailer will read it and use it to put the attachment.
	*/
	var $pdf_output;
	
	/*KONSTRUKTOR	
      - Creates the PDF
      - Sets the pagesize to A4
      - Sets milimeter-orientation
      - Sets the font arial, 12 Point.
	*/
	function bm_pdf()
	{
		// Code
		//Call FPDF Contructor
		$this->PDF_HTMLColor();
		//Add Page, does this belong here or to the assembler?
		$this->AddPage();
		$this->SetFont('Arial','',12);    
		//Set PDF-Metadata
		$this->SetTitle(BM_TXT_PDF_FILETITLE);
		$this->SetAuthor(BM_TXT_PDF_AUTHOR);
		$this->SetCreator(BM_TXT_PDF_CREATOR);
		
	}//END konstruktor bm_pdf
  
  /* header
    	Overwrites the FDPF-Method header beeing called for every new page.
    	This header has:
    		-cell, full pagewidth, line below
    		-Text 16-20 Points from the constant BM_TXT_HEADER_TEXT
    		-Bloodymary Logo as PNG at the right Side, beeing included via the function addlogo
    		- Some space below
  */
  function header()
	{
		//Basic Setting, Arial Bold, Size 18 Points
		$this->SetFont('Arial','B','18');
		$this->SetTextColor(BM_COLOR_BLAU_MITTEL);
		$this->SetFillColor(BM_COLOR_BLAU_HELL);
		$this->SetDrawColor(BM_COLOR_BLAU_MITTEL);
		$this->SetLineWidth(1.5);
		//Cell 100% Width, 30mm Hight, left bound, Border below, move to next line,fill
		$this->Cell(0,30,BM_TXT_PDF_HEADERTEXT,'B',1,'L',1);
		//Picture 75mm on the right (The picture is 106mm width and 26mm high) and 2mm above, equals centered in the header
		$this->addlogo(BM_LOGO_BM,92,12);
		$this->ln(40);
		//Reset values
		$this->SetFont('Arial','','12');
		//$this->SetTextColor(BM_COLOR_SCHWARZ);
		//$this->SetFillColor(BM_COLOR_WEISS);
	}//END header


	/* footer
	Overwrites the FDPF-Method footer beeing called for every new page.
    	This footer has:
    		- Space above
    		- Cell full width, line above
    		- version information left, text from BM_TXT_VERSION
    		- Pagenumber in the middle
    		- On the right side 3 powered by logos from FPDF, JPGRAPH, PHP-Mailer	
	*/
	function footer() 
	{
		//Move 20mm from bottom
		$this->SetY(-30);
		//Arial italic Size 10, Forecolor=black/Background=white
		$this->SetFont('Arial','I','10');
		$this->SetTextColor(BM_COLOR_SCHWARZ);
		$this->SetFillColor(BM_COLOR_WEISS);
		//The actual footer with version an creationdate
		//Cell 100mm Width, 30mm Heigth,no Text,border top,cursor right,left bound,no fill
		$this->MultiCell(0,5,BM_TXT_PDF_FOOTERTEXT,'T','L',0);
		//The Promofooter
		//3 Pictures beside each other, 2mm space between centered with the text of the multicell
		$this->addlogo(BM_LOGO_FPDF,113,-25);
		$this->addlogo(BM_LOGO_JPGRAPH,143,-25);
		$this->addlogo(BM_LOGO_PHPMAILER,173,-25);	
	}//END footer

	/* addheading
	   Adds a new centered heading
	   - set the cursor to the right position withn set_y
	   - create the cell, width=textwidth+3mm, centered, center the cell by using the formula pagewidth/titlewidth+3/2
	   - 5mm space below
	   - fontsize 18
	*/
	function addheading($text)
	{
		global $title;
		//Arial bold 15
		$this->SetFont('Arial','B',18);
	      //Calculate width of title and position
	      $w=$this->GetStringWidth($title)+6;
	      //$this->SetX((210-$w)/2);
		//Colors of frame, background and text
	      //$this->SetDrawColor(BM_COLOR_SCHWARZ);
	      //$this->SetFillColor(BM_COLOR_WEISS);
	      //$this->SetTextColor(BM_COLOR_SCHWARZ);
	      //Thickness of frame (2 mm)
	      //$this->SetLineWidth(1);
	      //Title Width $w mm, 10mm high, Border  below, Cursor Linebreak, centered, no fill
	      //$this->Cell(0,0,$text,0,1,'C',0);
	      $this->MultiCell(0,15,$text,'','C');
	      //Line break
	      $this->Ln(5);  
	}//END addheading

	/* addparagraph
	Adds a new paragraph
	- get the cursor position with get/set_y
	- create multicell, justified
	- space below
	- fontsize 12
	* PARAMETERS *
	$text		: Text to add
	$important	: renders the paragraph bold and a bit bigger
	*/
	function addparagraph($text,$important=FALSE)
	{
    		//Times 12, normal, if $important then bold and border with light blue background
    		if ($important)
	      {
			//Fett
			$this->SetFont('','B',10);
			//Colors of frame, background and text
		      $this->SetDrawColor(BM_COLOR_BLAU_DUNKEL);
		      $this->SetFillColor(BM_COLOR_BLAU_HELL);
		      $this->SetTextColor(BM_COLOR_SCHWARZ);
			//Multicell 160mm breit mit SetX 15mm nach Rechts rcken um Zentrierung zu erwirken
			//$this->SetX(15);
			$this->MultiCell(0,5,$text,1,'',1);
		} else {
			$this->SetFont('','',10);
			//Multicell 160mm breit mit SetX 15mm nach Rechts rcken um Zentrierung zu erwirken
			//$this->SetX(15);
			$this->MultiCell(0,5,$text);
		}
    		//Output justified text
    		//Line break, 2mm drunter platz
	      $this->Ln(5);
	
	}//END addparagraph

	/* addlogo
	Adds a predefined picture
		- navigate to the given position
		- paste picture
	The pictures are to be selected via BM_LOGO_ CONSTANT, there are
		- bloodymary
		- fpdf
		- jpgraph
		- phpmailer
	* PARAMETER *
	$selectlogo		: one of the CONSTANTS
	$x			: desired x-position
	$y			: desired y-position
	*/
	function addlogo($selectlogo,$x,$y)
	{
		/* ### DESIGN-NOTE ###
		Can one of the params be positive one negative, thus one refering to the upper
		left edge and one to the lower right edge?
		*/
		$this->SetXY($x,$y);
		/*
		Replace $x and $y with the absolute values, measured from upper left edge. They may be overwritten 
		because the are given ByVal and thus are just copies.
		*/
		$x=$this->GetX();
		$y=$this->GetY();
		
		//Convert BM_LOGO Constant to a filename
		/*define('BM_LOGO_BM',0);
		  define('BM_LOGO_FPDF',1);
	        define('BM_LOGO_JPGRAPH',2);
	        define('BM_LOGO_PHPMAILER',3);
	  	*/
		if ($selectlogo==BM_LOGO_BM)
			{ $filename=BM_TEMPLATE_LOGO_BM;}
		elseif ($selectlogo==BM_LOGO_FPDF)
			{ $filename=BM_TEMPLATE_LOGO_FPDF;}
		elseif ($selectlogo==BM_LOGO_JPGRAPH)
			{ $filename=BM_TEMPLATE_LOGO_JPGRAPH;}
		elseif ($selectlogo==BM_LOGO_PHPMAILER)
			{ $filename=BM_TEMPLATE_LOGO_PHPMAILER ;}
		else { $filename=BM_TEMPLATE_BM; }
		
		//Add the given image to the pdf
		$this->Image($filename,$x,$y);
	}//END addlogo
	
	/* addgraph
	Adds a predefined Bloodymary Graph
		- Navigate to the given postition
		- Calculate the size in pixels from the size in milimeters
		- Create the graph
		- Paste the Graph
		- Release the graph
	The graphs can be selected with the BM_GRAPHS constant. There are BM_GRAPH_FREESEXGRAPH and
	BM_GRAPH_FERTILITYGRAPH
	
	* PARAMETER *
	$selectgraph	: The BM_GRAPH constant to select the graph
	$width, height	: The desired size in milimeters
	*/
	function addgraph($selectgraph,$width,$height) 
	{
		//Import input data from the global scope
		global $bm_in_data;
		
		$aux=new auxtools();
		
		//Konstante zu Dateiname uebersetzen
		/*define('BM_GRAPH_FERTILITY',0);
	 	  define('BM_GRAPH_FREESEX',1);
	  	*/
  		//Convert heigth and width to from milimeters to pixels
	  	$pix_width=$aux->mmtopixel($width);
  		$pix_height=$aux->mmtopixel($height);
		
		if ($selectgraph==BM_GRAPH_FERTILITY)
			{ 
				//Create fertilitygraph
				$graph=new fertilitygrafik($pix_width,$pix_height);
				//Translate/Calculate the given inputdata and fill the graph-object with it
				$graph->translate_fertility_array($bm_in_data);
				//Set filename prefix for createoutput
				$filename_prefix=BM_TEMPLATE_FILENAME_PREFIX_FERTILITY;
			}
		elseif ($selectgraph==BM_GRAPH_FREESEX)
			{ 
				//Create freesexgraph
				$graph=new freesexgrafik($pix_width,$pix_height);
				//Translate/Calculate the given inputdata and fill the graph-object with it
				$graph->translate_freesex_array($bm_in_data);
				/*###DESIGN-NOTICE ###
				  Set filename prefix for createoutput.
				  Maybe createoutput could find the prefix itself (e.g. save it in the object) ot 
				  use a unique-id as filename
				*/
				//Set filename prefix for createoutput
				$filename_prefix=BM_TEMPLATE_FILENAME_PREFIX_FREESEX;
			}
		else { $aux->error(BM_OUT_SERVER_LOGIC_ERROR); }
				
		/*
		  Do the measures needed to embed the graph. 
		  Assertion: 
		  	-Constant was correct
		  	-There is an graph-object filled with data to output
		  	-The graph-object is ready to output
		*/		
		//Add the plot to the graph-object
		$graph->addplot();
		/*
		Output the graph to the servers harddisk, createoutput, the path of the created file is
		returned, ready to embed it.
		*/
		$filename=$graph->createoutput($filename_prefix);
		//Embed the graph
		//$this->Image($filename,$this->GetX(),$this->GetY());
		$this->Image($filename,0,$this->GetY());
		//Set the cursor below the image by setting it exactly the size of the image lower
		$this->SetY($this->GetY()+$height+15);
		/*
		The file created with createoutput can be released now. The return value of releaeoutput 
		is currently ignored, but could gain importance with bigger amounts of data - e.g. for 
		clean-up purposes
		*/
		$graph->releaseoutput();
	}//END addgraph
	
	/*createoutput
  	Returns the finished PDF as string and saves it in the pdf_output-property of this object.
  */
	function createoutput()
  	{
  		//Als String ausgeben
	  	/*### DEBUG ###
  		Output the PDF directly to the browser
	  	*/
			$pdf=$this->Output('','S');
	  	/* DEBUG ONLY
			$this->Output('test.pdf');
		*/
		$this->pdf_output=$pdf;
	  	return $pdf;
  }//END createoutput
  
   /* releaseoutput
  	Since the PDF is returned as string, this function does nothing, but is needed when the PDF-
  	Transfer is set to be file-based.
  */
  function releaseoutput()
  {
  	return true;
  
  }//END releaseoutput
  

}//END class bm_pdf

//### DEBUG ###
$deb=new auxtools();
//$backtrace=debug_backtrace();
//$deb->debug_out(__FILE__.' OK<br>'.'(in '.__FILE__.', Line: '.__LINE__.', Function: '.__FUNCTION__.')<br>');
?>